<?php

/*
This class processes the download
and meta-data of a certain photo

Functions:
- hasModifications
- processFile
- queryUrls
- download
- writeEXIF
- queryMeta
*/

class FlickrMovrPhoto {

	private $photoSet=null;

	private $id=-1;

	private $xml=null;

	private $urls=array();

	private $title="";
	private $description="";
	private $dateTaken="";
	private $lastUpdate="";
	private $tags=array();
	private $flickrLink="";
	private $latitude="";
	private $longitude="";

	private $flickrSizes=array("Original", "Large", "Medium", "Small", "Thumbnail", "Square");

	/*
	Constructor
	*/

	public function __construct($id, $lastUpdate, $photoSet) {

		$this->setID($id);
		$this->setPhotoSet($photoSet);

		/*
		if we are creating an xml database, try
		to retrieve an existing photo from the
		database
		*/

		if (CREATE_XML)
			$this->setXML($this->getFlickrXML()->getPhoto($this->getID()));

		/*
		did we have a modification since the last run?
		a modification it either a newer flickr
		last update timestamp or a non-existing
		file in case of DOWNLOAD_PHOTOS OR
		DOWNLOAD_THUMBNAILS

		hasModifications will also return true,
		if INCREMENTAL_BACKUP has been set to false
		*/

		if ($this->hasModifications($lastUpdate)) {

			/*
			handle downloads
			*/

			if (DOWNLOAD_PHOTOS)
				$this->processFile($this->getPhotoInRoot(), $this->getPhotoInSet(), 'photo');

			if (DOWNLOAD_THUMBNAILS)
				$this->processFile($this->getThumbnailInRoot(), $this->getThumbnailInSet(), 'thumbnail');

			/*
			we need the photo's meta to create
			the xml database and write EXIF-tags
			*/

			if (CREATE_XML || STORE_META_DATA_IN_EXIF)
				$querySuccess=$this->queryMeta();

			/*
			process photo meta-data
			*/

			if (CREATE_XML && $querySuccess)
				$this->getFlickrXML()->updatePhotoAttributes($this, $this->getPhotoSet()->getXML(), $this->getPhotoSet()->getPhotoLinksXML());

			/*
			process EXIF-tags with ExifTool
			*/

			if (STORE_META_DATA_IN_EXIF && $querySuccess) {
				$this->writeEXIF($this->getThumbnailInSet());
				$this->writeEXIF($this->getPhotoInSet());
			}

		}

	}

	/*
	setters and getters
	*/

	private function getFlickrXML() {
		return $this->getPhotoSet()->getFlickrXML();
	}

	private function setPhotoSet($photoset) {
		$this->photoset=$photoset;
	}

	public function getPhotoSet() {
		return $this->photoset;
	}

	private function setID($id) {
		$this->id=$id;
	}

	public function getID() {
		return $this->id;
	}

	private function setTitle($title) {
		$this->title=$title;
	}

	public function getTitle() {
		return $this->title;
	}

	private function setDescription($description) {
		$this->description=$description;
	}

	public function getDescription() {
		return $this->description;
	}

	private function setDateTaken($dateTaken) {
		$this->dateTaken=$dateTaken;
	}

	public function getDateTaken() {
		return $this->dateTaken;
	}

	private function setLastUpdate($lastUpdate) {
		$this->lastUpdate=$lastUpdate;
	}

	public function getLastUpdate() {
		return $this->lastUpdate;
	}

	private function addTag($tag) {
		array_push($this->tags, $tag);
	}

	public function getTags() {
		return $this->tags;
	}

	private function setFlickrLink($flickrLink) {
		$this->flickrLink=$flickrLink;
	}

	public function getFlickrLink() {
		return $this->flickrLink;
	}

	private function setLatitude($latitude) {
		$this->latitude=$latitude;
	}

	public function getLatitude() {
		return $this->latitude;
	}

	private function setLongitude($longitude) {
		$this->longitude=$longitude;
	}

	public function getLongitude() {
		return $this->longitude;
	}

	private function getFileName() {
		return $this->getID().'.jpg';
	}

	private function getPhotoInRoot() {
		return PHOTO_DIR.$this->getFileName();
	}

	private function getPhotoInSet() {
		return $this->getPhotoSet()->getDirectory().DIRECTORY_SEPARATOR.$this->getFileName();
	}

	private function getThumbnailInRoot() {
		return PHOTO_DIR.THUMBNAILS_DIR.DIRECTORY_SEPARATOR.$this->getFileName();
	}

	private function getThumbnailInSet() {
		return $this->getPhotoSet()->getThumbnailsDirectory().DIRECTORY_SEPARATOR.$this->getFileName();
	}

	private function setXML($xml) {
		$this->xml=$xml;
	}

	public function getXML() {
		return $this->xml;
	}

	private function getFlickrSizes() {
		return $this->flickrSizes;
	}

	/*
	checks if a photo has been modifed by comparing
	the timestamp of the xml database and
	the retrieved timestamp from the set's query
	*/

	private function hasModifications($flickrLastUpdate) {

		/*
		if we do not process an incremental backup
		we automatically return true
		*/

		if (!INCREMENTAL_BACKUP)
			return true;

		/*
		in case the xml-photo-node exists,
		we start the timestamp-comparison
		*/

		if (!empty($this->xml)) {

			/*
			the last modified attribute
			has to exist in the xml node
			*/

			$dbLastUpdate=-1;
			$tempLastUpdate=$this->getXML()->getAttribute('last_update');
			if (!empty($tempLastUpdate))
				$dbLastUpdate=$tempLastUpdate;

			/*
			do not process photo, if
			the last modified timestamp
			from last run is equal or smaller
			than the current timestamp

			to recover from errors of
			the last run, a photo/thumbnail will
			still be processed, if it doesn't
			exist in set(/thumbnails) directory

			moreover, we make sure that the
			retrieved photo_xml is in the
			current set; if not, we're
			handling a photo_link

			if the photo_link already exists,
			we can still skip
			*/

			if ($flickrLastUpdate<=$dbLastUpdate && (!DOWNLOAD_PHOTOS || (DOWNLOAD_PHOTOS && file_exists($this->getPhotoInSet()))) && (!DOWNLOAD_THUMBNAILS || (DOWNLOAD_PHOTOS && file_exists($this->getThumbnailInSet()))) && ($this->getXML()->parentNode->isSameNode($this->getPhotoSet()->getXML()) || (!$this->getXML()->parentNode->isSameNode($this->getPhotoSet()->getXML()) && $this->getFlickrXML()->photoLinkExists($this, $this->getPhotoSet()->getPhotoLinksXML()))) ) {
				Utils::echo_log('nothing changed for '.$this->getID());

				/*
				skip processing of this photo
				*/

				return false;
			}
		}

		/*
		if we land here, this photo needs to
		be re-processed
		*/

		return true;

	}

	/*
	handle download and copy of photo/thumbnail
	*/

	private function processFile($source, $destination, $type) {

		/*
		download photo/thumbnail if it does not exist
		*/

		if (!file_exists($source)) {
			if (empty($this->urls))
				$this->queryUrls();

			$this->download($source, $this->urls[$type], $type);
		}

		else
			Utils::echo_log($type.' '.$this->getID().' already exists - download skipped');

		/*
		copy photo/thumbnail from PHOTO_DIR to
		set's directory
		*/

		if (file_exists($source) && !file_exists($destination)) {
			if (@copy($source, $destination))
				Utils::echo_log('Copy '.$type.' '.$this->getFileName().' to set '.$this->getPhotoSet()->getDirectoryName());
			else
				Utils::echo_log('Failed to copy '.$type.' '.$this->getFileName().' to set '.$this->getPhotoSet()->getDirectoryName(), -1);
		}
	}

	/*
	retrieves a download_url from the photo's meta-data
	in decending order
	"Original" > "Large" > "Medium" > "Small" >
	"Thumbnail" > "Square"
	in other words: prefers larger to smaller size
	*/

	function queryUrls() {
		Utils::echo_log('Query flickr for url of '.$this->getID());

		/*
		query flickr for available photo sizes
		*/

		$rsp_obj=Utils::query_flickr_api('flickr.photos.getSizes', 'photo_id', $this->getID());

		if ($rsp_obj){

			if (array_key_exists('sizes', $rsp_obj)) {
				$sizes=$rsp_obj['sizes'];

				if (array_key_exists('size', $sizes)) {
					$sizes=$sizes['size'];

					$found=false;

					/*
					loop through the available sizes to
					find the largest one
					*/

					foreach($this->getFlickrSizes() as $flickr_size) {

						foreach($sizes as $size) {

							/*
							thumbnail-url (square)
							*/

							if ($size['label']=="Square") {
								$this->urls['thumbnail']=$size['source'];
							}

							/*
							original-url
							*/

							if (!$found && $size['label']==$flickr_size) {
								$found=true;
								$this->urls['photo']=$size['source'];
							}
			
						}
					}
				}

				else
					Utils::echo_log('Url information does not exist for photo '.$this->getID(), -1);

			}

			else
				Utils::echo_log('Failed flickr query', -1);
		}

		else
			Utils::echo_log('Failed flickr query', -1);

	}

	/*
	downloads a photo to the root of the photo directory
	*/

	function download($destinationFile, $photoURL, $type) {
		if ($photoURL!=null) {
			Utils::echo_log('Initiate download of '.$type.' '.$this->getId());

			$ret=Utils::cURLdownload($photoURL, $destinationFile);

			/*
			handle download replies
			*/

			if (!$ret)
				Utils::echo_log('Error processing download', -1);
			else
				Utils::echo_log('Successfully downloaded '.$ret);
		}

		else
			Utils::echo_log('The necessary url information does not exist for photo '.$this->getID(), -1);

	}

	/*
	writes flickr meta-data
	(e.g. title, GPS data, ...)
	to EXIF information of JPEG file
	and optionally set file date to creation date
	with ExifTool
	*/

	function writeEXIF($image) {
		Utils::echo_log('Write EXIF tags to '.$image);

		/*
		store flickr-title into
		EXIF-tag "ImageDescription",
		flickr-description into "UserComment",
		flickr-date_taken into "DateTimeOriginal" and "CreateDate"
		*/

		$exifTitle="";

		if (!empty($this->title)) {
			$exifTitle="-ImageDescription=\"".str_replace('"', '\'', $this->getTitle())."\"";
		}
			
		$exifComment="";

		if (!empty($this->description)) {
			$exifComment="-UserComment=\"".str_replace('"', '\'', $this->getDescription())."\"";
		}

		$exifOrigdate="";

		if (!empty($this->dateTaken)) {

			/*
			change date-format to exif-type
			yyyy:mm:dd hh:mm
			*/

			$dateTaken=str_replace('-', ':', $this->getDateTaken());
			$exifOrigdate="-DateTimeOriginal=\"".$dateTaken."\" -CreateDate=\"".$dateTaken."\"";

			if (UPDATE_FILE_DATE)
				$exifOrigdate.=" -FileModifyDate=\"".$dateTaken."\"";
		}

		$exifLocation="";

		if (!empty($this->latitude) && !empty($this->longitude)) {

			/*
			Conversion from flickr data:
			Latitude {+,-} = {N,S}
			Longitude {+,-} = {E,W}
			*/

			$latitudeRef="N";
			$longitudeRef="E";

			if ($this->getLatitude()<0)
				$latitudeRef="S";

			if ($this->getLongitude()<0)
				$longitudeRef="W";

			/*
			store flickr-location into
			geo-EXIF-tags
			GPSLatitudeRef, GPSLongitudeRef,
			GPSLatitude, GPSLongitude
			*/

			$exifLocation="-GPSLatitudeRef=".$latitudeRef." -GPSLongitudeRef=".$longitudeRef." -GPSLatitude=".abs($this->getLatitude())." -GPSLongitude=".abs($this->getLongitude());

		}

		/*
		store flickr-tags into EXIF-tag "keyword"
		*/

		$exifTags="";

		if (!empty($this->tags)) {
			foreach($this->getTags() as $tag) {
				$exifTags.='-keywords="'.str_replace('"', '\'', $tag).'" ';
			}
		}

		$log=Utils::exec_exif_tool($exifTitle." ".$exifComment." ".$exifOrigdate." ".$exifLocation." ".$exifTags, $image);

		if (strlen($log)>1)
			Utils::echo_log($image.' - ExifTool says:'.$log, -2);
	}

	/*
	retrieves a photo's meta from flickr
	- title
	- description
	- flickr photo-page url
	- date_taken
	- last_update
	- location[latitude, longitude]
	- tags[tag_1, .. , tag_n]
	*/

	function queryMeta() {
		Utils::echo_log('Query flickr for meta-tags of '.$this->getID());

		$rsp_obj=Utils::query_flickr_api('flickr.photos.getInfo', 'photo_id', $this->getID());

		if ($rsp_obj){

			if (array_key_exists('photo', $rsp_obj)) {
				$photo=$rsp_obj['photo'];

				if (array_key_exists('title', $photo)) {
					$this->setTitle(html_entity_decode($photo['title']["_content"], ENT_QUOTES));
				}

				if (array_key_exists('description', $photo)) {
					$this->setDescription(html_entity_decode($photo['description']["_content"], ENT_QUOTES));
				}

				if (array_key_exists('urls', $photo)) {
					$urls=$photo['urls'];
					if (array_key_exists('url', $urls)) {
						$url=$urls['url'];
						foreach ($url as $url_entry) {							if ($url_entry['type']=='photopage') {
								$this->setFlickrLink($url_entry['_content']);
							}
						}
					}
				}

				if (array_key_exists('dates', $photo)) {
					$dates=$photo['dates'];
					if (array_key_exists('taken', $dates)) {
						$this->setDateTaken($dates['taken']);
					}
					if (array_key_exists('lastupdate', $dates)) {
						$this->setLastUpdate($dates['lastupdate']);
					}
				}

				if (array_key_exists('tags', $photo)) {
					$photoTags=$photo['tags'];
					if (array_key_exists('tag', $photoTags)) {
						$tag=$photoTags['tag'];

						foreach($tag as $tag_entry) {
							$this->addTag(html_entity_decode($tag_entry['raw'], ENT_QUOTES));
						}
					}

				}

				if (array_key_exists('location', $photo)) {
					$locationArr=$photo['location'];

					if (array_key_exists('latitude', $locationArr) && array_key_exists('longitude', $locationArr)) {
						$this->setLatitude($locationArr['latitude']);
						$this->setLongitude($locationArr['longitude']);
					}
				}

				return true;

			}

			else
				Utils::echo_log('Failed flickr query', -1);
		}

		else
			Utils::echo_log('Failed flickr query', -1);

	return false;

	}


}
